package sk.upjs.js;

import java.util.List;

import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.BeanPropertySqlParameterSource;
import org.springframework.jdbc.core.simple.ParameterizedBeanPropertyRowMapper;
import org.springframework.jdbc.core.simple.SimpleJdbcInsert;
import org.springframework.jdbc.core.simple.SimpleJdbcTemplate;

/**
 * Databaza so studentami napevno
 *
 */
public class Database {
	/**
	 * JDBC ablna
	 */
	private SimpleJdbcTemplate jdbcTemplate;

	/**
	 * Mapova riadkov na tudentov
	 */
	private ParameterizedBeanPropertyRowMapper<Student> rowMapper 
		= ParameterizedBeanPropertyRowMapper.newInstance(Student.class); 
	
	/**
	 * Vrti zoznam vetkch tudentov v databze. 
	 */
	public List<Student> listStudents() {
		return jdbcTemplate.query("SELECT * FROM student", rowMapper);
	}

	/**
	 * Njde v databze tudenta s danm identifiktorom. Ak
	 * sa tak tudent nenjde, vrti <code>null</code>.
	 */
	public Student findById(Integer id) {
		List<Student> students = jdbcTemplate.query("SELECT * FROM student WHERE id = ?", rowMapper, id);
		if(students.isEmpty()) {
			return null;
		}
		return students.get(0);
	}

	/**
	 * Ulo alebo aktualizuje objekt tudenta. Ak m tudent
	 * identifiktor rovn <code>null</code>, znamen
	 * to intanciu, ktor v databze ete nie je. V opanom 
	 * prpade sa existujca intancia s takm ID nahrad
	 * novou intanciou s aktualizovanmi dtami. 
	 * 
	 * @param student
	 */
	public void saveOrUpdate(Student student) {
		if(student.getId() == null) {
			Number primaryKey = new SimpleJdbcInsert((JdbcTemplate)jdbcTemplate.getJdbcOperations())
				.withTableName("student")
				.usingColumns("meno", "priezvisko", "rocnik")
				.usingGeneratedKeyColumns("id")
				.executeAndReturnKey(new BeanPropertySqlParameterSource(student));
			student.setId(primaryKey.intValue());	
		} else {
			jdbcTemplate.update("UPDATE student SET meno = ?, priezvisko = ?, rocnik = ? WHERE id = ?", 
					student.getMeno(), student.getPriezvisko(), student.getRocnik(), student.getId());
		}	
	}

	/**
	 * Odstrni z databzy tudenta s danm identifiktorom.
	 * Intancia v parametri nemus ma vyplnen vetky daje, sta,
	 * ke m vyplnen identifiktor.
	 */ 
	public void removeStudent(Student student) {
		if(student.getId() != null) {
			jdbcTemplate.update("DELETE FROM student WHERE id = ?", student.getId());
		}
	}

	public void setJdbcTemplate(SimpleJdbcTemplate jdbcTemplate) {
		this.jdbcTemplate = jdbcTemplate;
	}
	
	
}
